/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */
#pragma once

#include <rtl/ustring.hxx>

inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_AREA
    = u"com.sun.star.chart2.AreaChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_BAR
    = u"com.sun.star.chart2.BarChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_COLUMN
    = u"com.sun.star.chart2.ColumnChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_LINE
    = u"com.sun.star.chart2.LineChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_SCATTER
    = u"com.sun.star.chart2.ScatterChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_PIE
    = u"com.sun.star.chart2.PieChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_NET
    = u"com.sun.star.chart2.NetChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_FILLED_NET
    = u"com.sun.star.chart2.FilledNetChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_CANDLESTICK
    = u"com.sun.star.chart2.CandleStickChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_BUBBLE
    = u"com.sun.star.chart2.BubbleChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_BOXWHISKER
    = u"com.sun.star.chart2.BoxWhiskerChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_CLUSTEREDCOLUMN
    = u"com.sun.star.chart2.ClusteredColumnChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_FUNNEL
    = u"com.sun.star.chart2.FunnelChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_PARETOLINE
    = u"com.sun.star.chart2.ParetoLineChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_REGIONMAP
    = u"com.sun.star.chart2.RegionMapChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_SUNBURST
    = u"com.sun.star.chart2.SunburstChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_TREEMAP
    = u"com.sun.star.chart2.TreemapChartType"_ustr;
inline constexpr OUString CHART2_SERVICE_NAME_CHARTTYPE_WATERFALL
    = u"com.sun.star.chart2.WaterfallChartType"_ustr;

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
